/* GNU POP3 - a small, fast, and efficient POP3 daemon
   Copyright (C) 1999 Jakob 'sparky' Kaivo <jkaivo@nodomainname.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#include "gnu-pop3d.h"

#define LINESLICE 160

/* Prints the header of a message plus a specified number of lines */

int
pop3_top (const char *arg)
{
  int i = 0, header = 1, done = 0, lbuf;
  int mesg, lines, empty_line = 1;
  char *mesgc, *linesc, *res;
  char buf[LINESLICE * sizeof (char)], lineend[3 * sizeof (char)];

  if (strlen (arg) == 0)
	return ERR_BAD_ARGS;

  if (state != TRANSACTION)
	return ERR_WRONG_STATE;

  mesgc = pop3_cmd (arg);
  linesc = pop3_args (arg);
  mesg = atoi (mesgc) - 1;
  lines = strlen (linesc) > 0 ? atoi (linesc) : -1;
  free (mesgc);
  free (linesc);

  if (lines < 0)
	return ERR_BAD_ARGS;

  if (pop3_mesg_exist (mesg) != OK)
	return ERR_NO_MESG;

  if (fsetpos (mbox, &(messages[mesg].header)) < 0)
	return ERR_FILEPOS;

  fprintf (ofile, "+OK\r\n");

  while ((res = fgets (buf, LINESLICE, mbox)) && !done && i <= lines)
	{
	  if (empty_line && !strncmp (buf, "From ", 5))
		done = 1;
	  else
		{
		  lineend[0] = '\0';
		  if ((buf[0] == '\n') || (buf[0] == '\r') || (buf[0] == '\0'))
			empty_line = 1;
		  else
			empty_line = 0;
		  if (empty_line == 1)
			header = 0;

		  lbuf = strlen (buf);
		  if (buf[lbuf - 1] == '\n')
			{
			  lbuf--;
			  strcpy (lineend, "\r\n");
			}
		  if (buf[lbuf - 1] == '\r')
			lbuf--;
		  if (buf[0] == '.')
			fprintf (ofile, ".%.*s%s", lbuf, buf, lineend);
		  else
			fprintf (ofile, "%.*s%s", lbuf, buf, lineend);

		  while ((res != NULL) && (lineend[0] == '\0' && strchr (buf, '\n') == NULL))
			{
			  res = fgets (buf, LINESLICE, mbox);
			  if (res == NULL)
				{
				  buf[0] = '\n';
				  buf[1] = '\0';
				}				/* last line in mailbox without \n */
			  lbuf = strlen (buf);
			  if (buf[lbuf - 1] == '\n')
				{
				  lbuf--;
				  strcpy (lineend, "\r\n");
				}
			  if ((lbuf > 0) && (buf[lbuf - 1] == '\r'))
				lbuf--;
			  fprintf (ofile, "%.*s%s", lbuf, buf, lineend);
			}
		  if (!header)
			i++;
		}
	}
  fprintf (ofile, ".\r\n");
  return OK;
}
